package control;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import model.*;
import model.fillingFunction.*;
import model.innerSolver.SolomonHeuristic;

public class Vrptw
{
	static private final String[] PATHS_FULL = new String[] { "c101", "c102", "c103", "c104", "c105", "c106", "c107", "c108", "c109", "c201", "c202",
			"c203", "c204", "c205", "c206", "c207", "c208", "r101", "r102", "r103", "r104", "r105", "r106", "r107", "r108", "r109", "r110", "r111", "r112",
			"r201", "r202", "r203", "r204", "r205", "r206", "r207", "r208", "r209", "r210", "r211", "rc101", "rc102", "rc103", "rc104", "rc105", "rc106",
			"rc107", "rc108", "rc201", "rc202", "rc203", "rc204", "rc205", "rc206", "rc207", "rc208" };

	/**
	 * @param args
	 */
	public static void main (String[] args)
	{
		try
		{
			String output_name = "solomon_comparision_vigo";
			PrintWriter pwr = new PrintWriter(new FileWriter("output" + output_name + ".txt", true));
			String[] pathsSpecific = new String[] { "c201" };

			for (String path : PATHS_FULL)
			{
				DataReader dr = new DataReader("./src/data/" + path + ".txt");
				Customers customers = new Customers(dr);
				// parameters
				boolean verbose = true;

				AbstractFillingFunction fillingFunction = new VigoFillingFunction(customers, 5.0);
				SolomonHeuristic sh = new SolomonHeuristic(customers, dr.getCapacity(), fillingFunction);
				Solution solution = sh.GetInitialSolution();
				
				TabuSearch ts = new TabuSearch(dr, solution, sh, fillingFunction, 9, 6, 4, 3, verbose);

				long startTime = System.currentTimeMillis();
				ts.execute();
				// Solution sol = ts.getOptimalSolution();

				String info = path + "\t&" + Math.floor((System.currentTimeMillis() - startTime) / 1000) + "\t&" + ts + "\\\\ \n";

				// System.out.println(sol);
				System.out.println(info);
				pwr.append(info);
				// pwr.print(info);
				pwr.flush();
			}

			pwr.close();
		} catch (IOException e)
		{
			e.printStackTrace();
		}
	}
}
