package model;

public class TabuList
{
	private double[] tabuList;
	private int length;
	private int pointer = 0;

	/**
	 * Initiates a TabuList of length 'length.'
	 * 
	 * @param length
	 *          Length of TabuList.
	 */
	public TabuList (int length)
	{
		tabuList = new double[length];
		this.length = length;
	}

	/**
	 * Resets the tabuList (that is overwrite the contents with 0.0).
	 */
	public void reset ()
	{
		for (int j = 0; j < length; j++)
			tabuList[j] = 0.0;
	}

	/**
	 * Adds move to tabu list.
	 * 
	 * @param move
	 *          Move to add.
	 * @return true if move is successfully added and false otherwise.
	 */
	public void addMove (double value)
	{
		if (contains(value))
			throw new IllegalArgumentException("TabuList for already contains the value: " + value);

		tabuList[pointer] = value;
		pointer = (pointer + 1) % length;
	}

	/**
	 * Checks if move is contained in TabuList.
	 * 
	 * @param solution
	 *          Solution to check for.
	 * @return True if solution is contained and false otherwise.
	 */
	public boolean contains (double value)
	{
		for (int i = 0; i < length; i++)
			if (tabuList[i] == value)
				return true;

		return false;
	}

	@Override
	public String toString ()
	{
		String s = "{";
		for (Double value : tabuList)
			s += value.toString() + ", ";

		return s = s.substring(0, s.length() - 2) + "}";
	}
}
