package model;

/**
 * A class containing individual tabu lists. 
 * @author jerome
 *
 */
public class TabuListContainer
{
	private TabuList[] container;
	private int maxNeighborhoodSize;

	/**
	 * Constructs a new TabuListContainer with 'maxNeighborhoodSize' tabu lists each of length
	 * 'length.'
	 * 
	 * @param length
	 *          The length of an individual tabu list in the container.
	 * @param maxNeighborhoodSize
	 *          Number of tabu lists in the container.
	 */
	public TabuListContainer (int length, int maxNeighborhoodSize)
	{
		container = new TabuList[maxNeighborhoodSize];
		this.maxNeighborhoodSize = maxNeighborhoodSize;

		for (int j = 0; j < maxNeighborhoodSize; j++)
			container[j] = new TabuList(length);
	}

	/**
	 * Resets all tabuList.
	 */
	public void resetAllTabuList ()
	{
		for (TabuList tl : container)
			tl.reset();
	}

	/**
	 * Adds a move to a given tabu list.
	 * 
	 * @param value
	 *          Value to add.
	 * @param neighborhood
	 *          Neighborhood whose tabu list to add the value to.
	 */
	public void addMove (double value, int neighborhood)
	{
		illegalArgument(neighborhood);

		container[neighborhood - 1].addMove(value);
	}

	/**
	 * Checks if a given tabu list contains a given value.
	 * 
	 * @param value
	 *          Value to check for.
	 * @param neighborhood
	 *          Neighborhood whose tabu list to check.
	 * @return
	 */
	public boolean contains (double value, int neighborhood)
	{
		illegalArgument(neighborhood);

		return container[neighborhood - 1].contains(value);
	}

	private void illegalArgument (int neighborhood)
	{
		if (neighborhood > maxNeighborhoodSize)
			throw new IllegalArgumentException("neighborhood must be smaller than " + maxNeighborhoodSize
					+ ", but was " + neighborhood);
	}
	
	@Override
	public String toString ()
	{
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < maxNeighborhoodSize; i++)
			sb.append("tabu list " + (i+1) + ": " + container[i].toString() + "\n");

		return sb.toString();
	}
}
