package model.fillingFunction;

import java.util.List;

import model.Customers;

public abstract class AbstractFillingFunction
{
	protected Customers customers;

	public AbstractFillingFunction (Customers customers)
	{
		this.customers = customers;
	}

	/**
	 * Calculates the filling function for a particular list of parameters.
	 * 
	 * @param customersVisiting
	 *          List of customers being visited by the vehicle.
	 * @param numbersOfCustomersVisiting
	 *          The number of customers being visited by the vehicle (given as a parameter for
	 *          performance reasons).
	 * @param arrivalTimes
	 *          The arrival times at the customers for the vehicle.
	 * @param capacityUsed
	 *          The mount of capacity already used by this particular vehicle.
	 * @return The value of the filling function for this vehicle.
	 */
	public abstract double calculate (List<Integer> customersVisiting,
			int numbersOfCustomersVisiting, List<Double> arrivalTimes, int capacityUsed);
	
	/**
	 * Provide the name of the filling function as string.
	 */
	@Override
	public abstract String toString ();
}
