package model.fillingFunction;

import java.util.List;

import model.Customers;

public class TunedFillingFunction extends AbstractFillingFunction
{

	/**
	 * 
	 * @param customers
	 */
	public TunedFillingFunction (Customers customers)
	{
		super(customers);
	}

	@Override
	public double calculate (List<Integer> customersVisiting, int numbersOfCustomersVisiting, List<Double> arrivalTimes, int capacityUsed)
	{
		double a_1 = capacityUsed / customers.getTotalDemand();
		// subtract 1 to adjust for depot.
		double a_2 = numbersOfCustomersVisiting - 1;
		double a_3 = customers.getNumberOfCustomers() - 1;
		double fillingFunction = a_1 - a_2 / a_3;

		double arrivalTimeFactor = 1.0;
		double avrArrivalTime = 0.0;
		double t_1;
		double t_2;

		double costFactor = 1.0;
		double maxDistance = 0.0;
		double d;

		for (int i = 0; i < numbersOfCustomersVisiting - 1; i++)
		{
			t_1 = arrivalTimes.get(i);
			t_2 = arrivalTimes.get(i + 1);

			arrivalTimeFactor *= (t_2 - t_1);
			avrArrivalTime += t_1;

			d = customers.Distance(customersVisiting.get(i), customersVisiting.get(i + 1));
			costFactor *= d;

			if (d > maxDistance)
				maxDistance = d;
		}

		avrArrivalTime = (avrArrivalTime + arrivalTimes.get(numbersOfCustomersVisiting - 1)) / numbersOfCustomersVisiting;

		return arrivalTimeFactor + fillingFunction + avrArrivalTime + maxDistance;
	}

	@Override
	public String toString ()
	{
		return "TunedFillingFunction";
	}
}
