package model.innerSolver;

import java.util.ArrayList;
import java.util.List;

import model.Customers;
import model.Solution;
import model.Vehicle;
import model.fillingFunction.AbstractFillingFunction;

public class BasicHeuristic extends AbstractInnerSolver
{

	public BasicHeuristic (Customers customers, int capacity, AbstractFillingFunction fillingFunction)
	{
		super(customers, capacity, fillingFunction);
	}

	@Override
	public Solution GetInitialSolution ()
	{
		List<Integer> workList = customers.getListOfCustomers();

		double distance_max = 0.0;
		int seedCustomer = 0;
		for (int customer : customers.getListOfCustomers())
		{
			if (customers.Distance(customer) > distance_max)
			{
				seedCustomer = customer;
				distance_max = customers.Distance(customer);
			}
		}

		Vehicle vehicle = new Vehicle(capacity, customers, fillingFunction);
		placeCustomer(vehicle, seedCustomer, workList);

		List<Vehicle> vehicles = new ArrayList<Vehicle>(20);

		while (!workList.isEmpty())
		{
			while (placeOptimalCustomer(workList, vehicle))
			{
			}

			vehicles.add(vehicle);
			vehicle = new Vehicle(capacity, customers, fillingFunction);
		}

		return new Solution(vehicles.toArray(new Vehicle[0]), customers.getNumberOfCustomers());
	}

	/**
	 * Finds the optimal customer with respect to the criteria described in Potvin-paper.
	 * 
	 * @param workList
	 *          Worklist to process, that is unassigned customers.
	 * @param vehicle
	 *          Vehicle to add customers to.
	 * @return Returns the customer to be added if there is a feasible customer. Returns -1 otherwise.
	 */
	private int findOptimalCustomer (List<Integer> workList, Vehicle vehicle)
	{
		for (int customer : workList)
			for (int j = 1; j <= vehicle.getNumbersOfCustomersVisiting(); j++)
				if (vehicle.canAssignCustomer(customer, j))
					return customer;

		return -1;
	}

	/**
	 * Finds and places the optimal customer according to the criteria described in Potvin-paper.
	 * 
	 * @param workList
	 *          Worklist to process, that is unassigned customers.
	 * @param vehicle
	 *          Vehicle to add customers to.
	 * 
	 * @return Returns true if it is still feasible to assign further customer to the particular
	 *         vehicle and false otherwise.
	 */
	private boolean placeOptimalCustomer (List<Integer> workList, Vehicle vehicle)
	{
		int nextCustomer = findOptimalCustomer(workList, vehicle);
		if (nextCustomer != -1)
			placeCustomer(vehicle, nextCustomer, workList);
		return nextCustomer != -1;
	}

	/**
	 * Adds customer to vehicle and remove him/her from the worklist.
	 * 
	 * @param vehicle
	 *          Vehicle which to add customer to.
	 * @param seedCustomer
	 *          Customer to be added/removed.
	 * @param workList
	 *          Worklist to remove customer from.
	 */
	private void placeCustomer (Vehicle vehicle, int customer, List<Integer> workList)
	{
		if (vehicle.AssignCustomer(customer))
			workList.remove((Object) customer);
	}

	@Override
	public Solution GetSolution (List<Integer> customers)
	{
		// TODO Auto-generated method stub
		return null;
	}

}
