package test;

import java.util.ArrayList;
import java.util.List;

import model.*;
import model.fillingFunction.AbstractFillingFunction;
import model.fillingFunction.InitialFillingFunction;

abstract public class AbstractTestCase
{
	protected DataReader drSmall;
	protected DataReader drMSmall;
	protected DataReader drMedium;
	protected DataReader drBig;
	protected DataReader drFull;
	protected Customers customersSmall;
	protected Customers customersMSmall;
	protected Customers customersMedium;
	protected Customers customersBig;
	protected Customers customersFull;
	protected AbstractFillingFunction fillingFunction;
	protected Vehicle vehicle;
	protected Vehicle vehicle1;
	protected Vehicle vehicle2;
	protected Solution solution;
	protected TabuList tabuList;
	protected TabuListContainer tabuListContainer;

	protected long startTime = 0;

	protected double precision = 1E-15;

	protected void baseSetUp () throws Exception
	{
		drSmall = new DataReader(
				"./src/unittest_data/small_data.txt");
		customersSmall = new Customers(drSmall);

		drMSmall = new DataReader(
		"./src/unittest_data/msmall_data.txt");
customersMSmall = new Customers(drMSmall);

		drMedium = new DataReader(
				"./src/unittest_data/medium_data.txt");
		customersMedium = new Customers(drMedium);

		drBig = new DataReader(
				"./src/unittest_data/big_data.txt");
		customersBig = new Customers(drBig);

		drFull = new DataReader(
				"./src/data/r101.txt");
		customersFull = new Customers(drFull);
		
		fillingFunction = new InitialFillingFunction(customersFull, 0.5);
		
		vehicle = new Vehicle(drFull.getCapacity(), customersFull, fillingFunction);
		
		Vehicle[] vehicles = new Vehicle[2];
		vehicle1 = new Vehicle(200, customersFull, fillingFunction);
		vehicle2 = new Vehicle(200, customersFull, fillingFunction);
		vehicle1.AssignCustomer(2);
		vehicle1.AssignCustomer(3);
		vehicle1.AssignCustomer(13);
		vehicle2.AssignCustomer(4);
		vehicles[0] = vehicle1;
		vehicles[1] = vehicle2;

		solution = new Solution(vehicles, customersFull.getNumberOfCustomers());

		tabuList = new TabuList(3);
		tabuListContainer = new TabuListContainer(5, 4);
	}

	/**
	 * Adds customer 2, 3 and 4.
	 * 
	 * @return
	 */
	protected List<Integer> AddCustomers ()
	{
		List<Integer> listOfCustomers = new ArrayList<Integer>();

		listOfCustomers.add(2);
		listOfCustomers.add(3);
		listOfCustomers.add(13);

		vehicle.AssignCustomer(listOfCustomers);
		return listOfCustomers;
	}

	protected void startStopWatch ()
	{
		startTime = System.currentTimeMillis();
	}

	protected void stopStopWatch ()
	{
		System.out.println("Runtime: " + (System.currentTimeMillis() - startTime) + "ms.");
	}

}
