package test;

import junit.framework.Assert;
import model.*;

import org.junit.Before;
import org.junit.Test;

public class Customers_Test extends AbstractTestCase
{
	private Customers customers;
	private DataReader dr;

	@Before
	public void setUp () throws Exception
	{
		baseSetUp();
		dr = new DataReader(
				"./src/data/r101.txt");
		customers = new Customers(dr);
	}

	@Test
	public void testCalculateDistanceReflexive ()
	{
		int j = 0;
		for (; j < customers.getNumberOfCustomers(); j++)
			Assert.assertEquals(0.0, customers.Distance(j, j));

		Assert.assertEquals(101, j);
	}

	@Test
	public void testCalculateDistance ()
	{
		Assert.assertEquals(18.0, customers.Distance(0, 2));
		Assert.assertEquals(25.0, customers.Distance(3, 4));
		Assert.assertEquals(Math.sqrt(80.0), customers.Distance(4, 72), precision);
		Assert.assertEquals(0.0, customers.Distance(4, 72) - customers.Distance(72, 4), precision);
		Assert.assertEquals(0.0, customers.Distance(100, 0) - customers.Distance(0, 100), precision);
		Assert.assertEquals(Math.sqrt(578.0), customers.Distance(0, 100), precision);
		Assert.assertEquals(Math.sqrt(10.0), customers.Distance(98, 100), precision);
	}

	@Test
	public void testGetDistanceToDepot ()
	{
		Assert.assertEquals(customers.Distance(0, 100), customers.Distance(100));
	}

	@Test
	public void testGetNumberOfCustomers ()
	{
		Assert.assertEquals(101, customers.getNumberOfCustomers());
	}

	@Test
	public void testGetCoordinates ()
	{
		// Depot
		Assert.assertEquals(35, customers.getCustomerCoordinates(0)[0]);
		Assert.assertEquals(35, customers.getCustomerCoordinates(0)[1]);

		// Customers
		Assert.assertEquals(18, customers.getCustomerCoordinates(100)[0]);
		Assert.assertEquals(18, customers.getCustomerCoordinates(100)[1]);
		Assert.assertEquals(2, customers.getCustomerCoordinates(46)[0]);
		Assert.assertEquals(48, customers.getCustomerCoordinates(46)[1]);
	}

	@Test
	public void testGetDemand ()
	{
		Assert.assertEquals(0, customers.getDemand(0));
		Assert.assertEquals(10, customers.getDemand(1));
		Assert.assertEquals(17, customers.getDemand(100));
		Assert.assertEquals(1, customers.getDemand(46));
	}

	@Test
	public void testGetReadyTime ()
	{
		Assert.assertEquals(0, customers.getReadyTime(0));
		Assert.assertEquals(161, customers.getReadyTime(1));
		Assert.assertEquals(185, customers.getReadyTime(100));
		Assert.assertEquals(117, customers.getReadyTime(46));
	}

	@Test
	public void testGetDueDate ()
	{
		Assert.assertEquals(230, customers.getDueDate(0));
		Assert.assertEquals(171, customers.getDueDate(1));
		Assert.assertEquals(195, customers.getDueDate(100));
		Assert.assertEquals(127, customers.getDueDate(46));
	}

	@Test
	public void testGetServiceTime ()
	{
		Assert.assertEquals(0, customers.getServiceTime(0));
		Assert.assertEquals(10, customers.getServiceTime(1));
		Assert.assertEquals(10, customers.getServiceTime(100));
		Assert.assertEquals(10, customers.getServiceTime(46));
	}

	@Test
	public void testGetTotalDemand ()
	{
		Assert.assertEquals(1458, customers.getTotalDemand());
		int aggregatedDemand = 0;
		for (int customer : customers.getListOfCustomers())
			aggregatedDemand += customers.getDemand(customer);

		Assert.assertEquals(aggregatedDemand, customers.getTotalDemand());
	}
}
