package test;

import junit.framework.Assert;
import model.Customers;
import model.DataReader;
import model.Solution;
import model.TabuSearch;
import model.Vehicle;
import model.fillingFunction.AbstractFillingFunction;
import model.fillingFunction.InitialFillingFunction;
import model.innerSolver.SolomonHeuristic;

import org.junit.Before;
import org.junit.Test;

public class TabuSearch_Test extends AbstractTestCase
{

	@Before
	public void setUp () throws Exception
	{
		baseSetUp();
	}

	@Test
	public void test20SeparateCustomers ()
	{
		AbstractFillingFunction fillingFunction = new InitialFillingFunction(customersMedium, 0.5);
		SolomonHeuristic solomonHeuristicMedium = new SolomonHeuristic(customersMedium, drMedium.getCapacity(), fillingFunction);
		Solution solution = solomonHeuristicMedium.GetInitialSolution();
		solution = solution.visitWithSeparateVehicles(solution.getCustomersVisited(), fillingFunction);
		TabuSearch tabuSearch = new TabuSearch(drMedium, solution, solomonHeuristicMedium, fillingFunction, 7, 2, 3, 2, false);

		tabuSearch.execute();
		Solution sol = tabuSearch.getOptimalSolution();

		Assert.assertTrue(sol.isValid());
		Assert.assertTrue(solomonHeuristicMedium.GetInitialSolution().numberOfVehiclesUsed() - sol.numberOfVehiclesUsed() >= 0);
	}

	@Test
	public void visitAllWithOneVehicle ()
	{
		DataReader dr = new DataReader("./src/unittest_data/visit_all_with_one_vehicle.txt");
		Customers customers = new Customers(dr);
		AbstractFillingFunction fillingFunction = new InitialFillingFunction(customers, 0.5);
		SolomonHeuristic sh = new SolomonHeuristic(customers, dr.getCapacity(), fillingFunction);
		Solution solution = sh.GetInitialSolution();
		solution = solution.visitWithSeparateVehicles(solution.getCustomersVisited(), fillingFunction);
		TabuSearch ts = new TabuSearch(dr, solution, sh, fillingFunction, 5, 2, 3, 2, false);

		ts.execute();
		Solution sol = ts.getOptimalSolution();

		Vehicle v = new Vehicle(dr.getCapacity(), customers, fillingFunction);
		for (int i = 1; i < 5; i++)
			v.AssignCustomer(i);

		Vehicle[] vehicles = new Vehicle[] { v };
		Solution expected = new Solution(vehicles, customers.getNumberOfCustomers());

		Assert.assertTrue(sol.equals(expected));
		System.out.println(sol);
	}

	@Test
	public void twoVehicles ()
	{
		DataReader dr = new DataReader("./src/unittest_data/two_vehicles.txt");
		Customers customers = new Customers(dr);
		AbstractFillingFunction fillingFunction = new InitialFillingFunction(customers, 0.5);
		SolomonHeuristic sh = new SolomonHeuristic(customers, dr.getCapacity(), fillingFunction);
		Solution solution = sh.GetInitialSolution();
		solution = solution.visitWithSeparateVehicles(solution.getCustomersVisited(), fillingFunction);
		TabuSearch ts = new TabuSearch(dr, solution, sh, fillingFunction, 5, 2, 3, 2, false);

		ts.execute();
		Solution sol = ts.getOptimalSolution();

		Vehicle v = new Vehicle(dr.getCapacity(), customers, fillingFunction);
		Vehicle v2 = new Vehicle(dr.getCapacity(), customers, fillingFunction);

		for (int i = 1; i < 4; i++)
		{
			v.AssignCustomer(i);
			v2.AssignCustomer(i + 3);
		}

		Vehicle[] vehicles = new Vehicle[] { v, v2 };
		Solution expected = new Solution(vehicles, customers.getNumberOfCustomers());

		Assert.assertTrue(sol.equals(expected));
	}

	@Test
	public void customer28And30 ()
	{
		DataReader dr = new DataReader("./src/unittest_data/small_data28And30.txt");
		Customers customers = new Customers(dr);
		AbstractFillingFunction fillingFunction = new InitialFillingFunction(customers, 0.5);
		SolomonHeuristic sh = new SolomonHeuristic(customers, dr.getCapacity(), fillingFunction);
		Solution solution = sh.GetInitialSolution();
		solution = solution.visitWithSeparateVehicles(solution.getCustomersVisited(), fillingFunction);
		TabuSearch ts = new TabuSearch(dr, solution, sh, fillingFunction, 5, 2, 3, 2, false);

		ts.execute();
		Solution sol = ts.getOptimalSolution();
		Assert.assertTrue(sol.isValid());
	}

	@Test
	public void customers50 ()
	{
		DataReader dr = new DataReader("./src/unittest_data/50_customers.txt");
		Customers customers = new Customers(dr);
		AbstractFillingFunction fillingFunction = new InitialFillingFunction(customers, 0.5);
		SolomonHeuristic sh = new SolomonHeuristic(customers, dr.getCapacity(), fillingFunction);
		Solution solution = sh.GetInitialSolution();
		solution = solution.visitWithSeparateVehicles(solution.getCustomersVisited(), fillingFunction);
		TabuSearch ts = new TabuSearch(dr, solution, sh, fillingFunction, 5, 2, 3, 2, false);

		startStopWatch();
		ts.execute();
		Solution sol = ts.getOptimalSolution();
		stopStopWatch();

		Assert.assertTrue(sol.isValid());
		Assert.assertEquals(12, sol.numberOfVehiclesUsed());
	}

	@Test
	public void testSmall ()
	{
		AbstractFillingFunction fillingFunction = new InitialFillingFunction(customersSmall, 0.5);
		SolomonHeuristic solomonHeuristicSmall = new SolomonHeuristic(customersSmall, drSmall.getCapacity(), fillingFunction);
		Solution solution = solomonHeuristicSmall.GetInitialSolution();
		TabuSearch tabuSearch = new TabuSearch(drSmall, solution, solomonHeuristicSmall, fillingFunction, 7, 4, 3, 2, false);
		tabuSearch.execute();
		Solution sol = tabuSearch.getOptimalSolution();
		Assert.assertTrue(sol.isValid());
		Solution sol2 = solomonHeuristicSmall.GetInitialSolution();
		Assert.assertTrue(sol.equals(sol2));
	}

	@Test
	public void testMediumSmall ()
	{
		AbstractFillingFunction fillingFunction = new InitialFillingFunction(customersSmall, 0.5);
		SolomonHeuristic solomonHeuristicSmall = new SolomonHeuristic(customersMSmall, drMSmall.getCapacity(), fillingFunction);
		Solution solution = solomonHeuristicSmall.GetInitialSolution();
		solution = solution.visitWithSeparateVehicles(solution.getCustomersVisited(), fillingFunction);
		TabuSearch tabuSearch = new TabuSearch(drMSmall, solution, solomonHeuristicSmall, fillingFunction, 7, 2, 3, 2, false);
		startStopWatch();
		tabuSearch.execute();
		Solution sol = tabuSearch.getOptimalSolution();
		stopStopWatch();
		System.out.println(sol);
		Assert.assertTrue(sol.isValid());
		Solution sol2 = solomonHeuristicSmall.GetInitialSolution();
		System.out.println(sol2);
	}

	// @Test
	public void testBig ()
	{
		AbstractFillingFunction fillingFunction = new InitialFillingFunction(customersBig, 0.5);
		SolomonHeuristic solomonHeuristic = new SolomonHeuristic(customersBig, drBig.getCapacity(), fillingFunction);
		Solution solution = solomonHeuristic.GetInitialSolution();
		TabuSearch tabuSearch = new TabuSearch(drBig, solution, solomonHeuristic, fillingFunction, 5, 2, 3, 2, false);
		startStopWatch();
		tabuSearch.execute();
		Solution sol = tabuSearch.getOptimalSolution();
		stopStopWatch();
		System.out.println(sol);
		Assert.assertTrue(sol.isValid());
		Solution sol2 = solomonHeuristic.GetInitialSolution();
		System.out.println(sol2);
	}

	// @Test
	public void testFull ()
	{
		AbstractFillingFunction fillingFunction = new InitialFillingFunction(customersFull, 0.5);
		SolomonHeuristic solomonHeuristic = new SolomonHeuristic(customersFull, drFull.getCapacity(), fillingFunction);
		Solution solution = solomonHeuristic.GetInitialSolution();
		TabuSearch tabuSearch = new TabuSearch(drFull, solution, solomonHeuristic, fillingFunction, 5, 2, 3, 2, false);
		startStopWatch();
		tabuSearch.execute();
		Solution sol = tabuSearch.getOptimalSolution();
		stopStopWatch();
		System.out.println(sol);
		Assert.assertTrue(sol.isValid());
		Solution sol2 = solomonHeuristic.GetInitialSolution();
		System.out.println(sol2);
	}
}
