package test;

import model.Utilities;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class Utilities_Test extends AbstractTestCase
{

	@Before
	public void setUp () throws Exception
	{
		baseSetUp();
	}

	@Test
	public void testBinom ()
	{
		Assert.assertEquals((int) 10, Utilities.binom(5, 2));
		Assert.assertEquals((int) 3, Utilities.binom(3, 1));
		Assert.assertEquals(Utilities.binom(11, 7), Utilities.binom(11, 4));

		Assert.assertEquals(1, Utilities.binom(11, 0));
		Assert.assertEquals(0, Utilities.binom(0, 2));
		Assert.assertEquals(0, Utilities.binom(5, 6));

		Assert.assertEquals(Utilities.binom(13, 2), Utilities.binom(13, 11));
		Assert.assertEquals(78, Utilities.binom(13, 11));
		Assert.assertEquals(2311801440l, Utilities.binom(40, 11));

		for (int k = 1; k <= 7; k++)
			Assert.assertEquals(Utilities.binom(14, k), Utilities.binom(14, 14 - k));
	}

	@Test
	public void testMask ()
	{
		for (int n = 1; n <= 15; n++)
		{
			byte[] mask = new byte[n];

			int i = 1;
			while (Utilities.next(mask, n))
			{
				i++;
			}
			Assert.assertEquals((int) Math.pow(2, n), i);

			for (int k = 1; k <= n; k++)
			{
				mask = new byte[n];
				i = 0;
				while (Utilities.next(mask, n, k))
				{
					i++;
				}
				Assert.assertEquals(Utilities.binom(n, k), i);
			}
		}
	}

	@Test
	public void testMaskScalability ()
	{
		startStopWatch();
		int n = 25000;
		int k = 1;
		byte[] mask = new byte[n];
		int i = 0;
		while (Utilities.next(mask, n, k))
		{
			i++;
		}
		Assert.assertEquals(n, i);
		Assert.assertTrue(System.currentTimeMillis() - startTime <= 500.0);

		startStopWatch();
		n = 20;
		for (k = 1; k <= 2; k++)
		{
			mask = new byte[n];
			while (Utilities.next(mask, n, k))
			{
			}
			Assert.assertTrue(System.currentTimeMillis() - startTime <= 500.0);
		}
	}

	@Test
	public void testVigoMask ()
	{
		int n = 40;
		int k = 4;
		byte[] kb = new byte[n];
		int i = 0;
		while (Utilities.nextKT(kb, k, 0, n))
		{
			i++;
		}
	}

	@Test
	public void testDetermineDthLowestVehicle ()
	{
		for (int i = 1; i < 100; i++)
			Utilities.determineTargetVehicle(solution, i);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testForExceptionOnDthLowestVehicle ()
	{
		Utilities.determineTargetVehicle(solution, 0);
	}
}
