package test;

import java.util.ArrayList;
import java.util.List;

import model.Vehicle;
import model.fillingFunction.AbstractFillingFunction;
import model.fillingFunction.InitialFillingFunction;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class Vehicle_Test extends AbstractTestCase
{
	@Before
	public void setUp () throws Exception
	{
		baseSetUp();
	}

	@Test
	public void testGetDepot ()
	{
		Assert.assertEquals((int) 0, (int) vehicle.getCustomersVisiting().get(0));
	}

	@Test
	public void testConstructorWithCustomer ()
	{
		Vehicle vehicle = new Vehicle(200, customersFull, new InitialFillingFunction(customersFull, 0.5), 2);
		Assert.assertTrue(vehicle.ContainsCustomer(2));
	}

	@Test
	public void testGetCustomersVisiting ()
	{
		Assert.assertEquals(1, vehicle.getCustomersVisiting().size());
	}

	@Test
	public void testGetCustomersVisiting2 ()
	{
		AddFirstFiveCustomers();
		Assert.assertEquals(2, vehicle.getCustomersVisiting().size());
	}

	@Test
	public void testGetCustomersVisiting3 ()
	{
		AddCustomers();
		Assert.assertEquals(4, vehicle.getCustomersVisiting().size());
	}

	@Test
	public void testCanAssignCustomer ()
	{
		Assert.assertTrue(vehicle.canAssignCustomer(1, -1));
		Assert.assertTrue(vehicle.AssignCustomer(1));
		Assert.assertFalse(vehicle.canAssignCustomer(1, -1));
		Assert.assertFalse(vehicle.canAssignCustomer(2, -1));
	}

	@Test
	public void testCanAssignCustomer2 ()
	{
		// Customer 65 is the customer farthest away from the depot in the dataset.

		Vehicle v = new Vehicle(drFull.getCapacity(), customersFull, new InitialFillingFunction(customersFull, 0.5));
		Assert.assertFalse(v.canAssignCustomer(65, 0));
		Assert.assertTrue(v.canAssignCustomer(65, 1));
		v.AssignCustomer(65);
		Assert.assertTrue(v.canAssignCustomer(1, 2));
		Assert.assertTrue(v.canAssignCustomer(66, 2));
	}

	@Test
	public void testCapacity ()
	{
		Assert.assertEquals(200, vehicle.getCapacity());
	}

	@Test
	public void testGetCapacityUsed ()
	{
		Assert.assertEquals(0, vehicle.getCapacityUsed());
	}

	@Test
	public void testGetCapacityUsed2 ()
	{
		AddCustomers();
		Assert.assertEquals(43, vehicle.getCapacityUsed());
	}

	@Test
	public void testGetSpareCapacityUsed ()
	{
		AddCustomers();
		Assert.assertEquals(157, vehicle.getSpareCapacity());
	}

	@Test
	public void testGetSpareCapacityUsed2 ()
	{
		Assert.assertEquals(vehicle.getCapacity(), vehicle.getCapacityUsed()
				+ vehicle.getSpareCapacity());
		AddCustomers();
		Assert.assertEquals(vehicle.getCapacity(), vehicle.getCapacityUsed()
				+ vehicle.getSpareCapacity());
	}

	@Test
	public void testAssignCustomer ()
	{
		vehicle.AssignCustomer(1);
		Assert.assertEquals((int) 1, (int) vehicle.getCustomersVisiting().get(1));
	}

	@Test
	public void testAssignCustomer2 ()
	{
		vehicle.AssignCustomer(1, 1);
		Assert.assertEquals((int) 1, (int) vehicle.getCustomersVisiting().get(1));
	}

	@Test
	public void testAssignMultipleCustomers ()
	{
		AddCustomers();

		Assert.assertEquals(0, (int) vehicle.getCustomersVisiting().get(0));
		Assert.assertEquals(2, (int) vehicle.getCustomersVisiting().get(1));
		Assert.assertEquals(3, (int) vehicle.getCustomersVisiting().get(2));
		Assert.assertEquals(13, (int) vehicle.getCustomersVisiting().get(3));
	}

	@Test
	public void testAssignMultipleCustomers2 ()
	{
		vehicle.AssignCustomer(1);
		vehicle.AssignCustomer(2);
		vehicle.AssignCustomer(3);
		Assert.assertEquals(2, vehicle.getCustomersVisiting().size());
	}

	@Test
	public void testAssignCustomerAtIndex ()
	{
		vehicle.AssignCustomer(2);
		vehicle.AssignCustomer(13);
		vehicle.AssignCustomer(3, 2);// Customer 3 at index 2

		int[] tempCust = { 0, 2, 3, 13 };

		for (int j = 0; j <= 3; j++)
			Assert.assertEquals(tempCust[j], (int) vehicle.getCustomersVisiting().get(j));
	}

	@Test(expected = IllegalArgumentException.class)
	public void testAssignCustomerAtTooBigIndex ()
	{
		vehicle.AssignCustomer(10, 10);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testAssignCustomerAtNegativeIndex ()
	{
		Assert.assertFalse(vehicle.AssignCustomer(10, -5));
	}

	@Test
	public void testAssignCustomerAtNegativeIndex2 ()
	{
		vehicle.AssignCustomer(10, -1);
		Assert.assertEquals(10, (int) vehicle.getCustomersVisiting().get(1));
	}

	@Test(expected = IllegalArgumentException.class)
	public void testAssignCutomerAtIndexZero ()
	{
		vehicle.AssignCustomer(84, 0);
	}

	@Test
	public void testAssignCutomerTwice ()
	{
		vehicle.AssignCustomer(84);
		Assert.assertFalse(vehicle.AssignCustomer(84));
	}

	@Test
	public void testRemoveOnlyCustomer ()
	{
		vehicle.AssignCustomer(84);
		vehicle.RemoveCustomer(84);
		Assert.assertEquals(1, vehicle.getCustomersVisiting().size());
	}

	@Test
	public void testRemoveCustomer2 ()
	{
		Assert.assertTrue(vehicle.AssignCustomer(84));
		Assert.assertEquals(2, vehicle.getCustomersVisiting().size());
		Assert.assertEquals(2, vehicle.getArrivalTimes().size());
		Assert.assertEquals(101.0, (double) vehicle.getArrivalTimes().get(0), precision);
		Assert.assertEquals(111.0 + Math.sqrt(592), (double) vehicle.getArrivalTimes().get(1),
				precision);
		Assert.assertEquals(2, vehicle.getNumbersOfCustomersVisiting());

		Assert.assertTrue(vehicle.RemoveCustomer(84));
		Assert.assertEquals(1, vehicle.getCustomersVisiting().size());
		Assert.assertEquals(1, vehicle.getArrivalTimes().size());
		Assert.assertEquals(0.0, (double) vehicle.getArrivalTimes().get(0), precision);
		Assert.assertEquals(1, vehicle.getNumbersOfCustomersVisiting());
	}

	@Test(expected = IllegalArgumentException.class)
	public void testRemoveDepot ()
	{
		vehicle.RemoveCustomer(0);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testRemoveNonExistentCustomer ()
	{
		AddCustomers();
		vehicle.RemoveCustomer(12);
	}

	@Test
	public void testRemoveCustomers ()
	{
		List<Integer> listOfCustomers = AddCustomers();
		vehicle.RemoveCustomer(listOfCustomers);
		Assert.assertEquals(1, vehicle.getCustomersVisiting().size());
		Assert.assertEquals(0, (int) vehicle.getCustomersVisiting().get(0));
	}

	@Test
	public void testCostOneCustomer ()
	{
		vehicle.AssignCustomer(1);
		Assert.assertEquals(2 * customersFull.Distance(0, 1), vehicle.getCost(), precision);
	}

	@Test
	public void testCostIdemPotent ()
	{
		vehicle.AssignCustomer(1);
		double cost = vehicle.getCost();
		Assert.assertEquals(cost, vehicle.getCost(), precision);
	}

	@Test
	public void testCostMultipleCustomer ()
	{
		AddCustomers();
		double distance = 18.0 + Math.sqrt(1184.0) + Math.sqrt(1025.0) + Math.sqrt(125.0);

		Assert.assertEquals(distance, vehicle.getCost(), precision);
	}

	@Test
	public void testCostWithRemoveMultipleCustomer ()
	{
		List<Integer> listOfCustomers = AddCustomers();
		vehicle.RemoveCustomer(listOfCustomers);

		Assert.assertEquals(0, vehicle.getCost(), precision);
	}

	@Test
	public void testCostWithRemoveMultipleCustomer2 ()
	{
		AddCustomers();

		vehicle.RemoveCustomer(13);
		Assert.assertEquals(18.0 + Math.sqrt(1184) + Math.sqrt(500), vehicle.getCost(), precision);

		vehicle.RemoveCustomer(3);
		Assert.assertEquals(36, vehicle.getCost(), precision);
	}

	@Test
	public void testContainsCustomer ()
	{
		AddCustomers();
		Assert.assertTrue(vehicle.ContainsCustomer(2));
		Assert.assertTrue(vehicle.ContainsCustomer(3));
		Assert.assertTrue(vehicle.ContainsCustomer(13));

		Assert.assertFalse(vehicle.ContainsCustomer(1));
	}

	@Test
	public void testClearListAndSetCustomersVisiting ()
	{
		List<Integer> customers = AddCustomers();
		vehicle.ClearListOfCustomersVisiting();
		Assert.assertEquals(1, vehicle.getCustomersVisiting().size());
		vehicle.setCustomersVisiting(customers);
		Assert.assertTrue(customers.equals(vehicle.getCustomersVisiting()));
	}

	@Test
	public void testArrivalTimes ()
	{
		AddCustomers();
		List<Double> arrivalTimes = vehicle.getArrivalTimes();
		Assert.assertEquals(4, arrivalTimes.size());
		Assert.assertEquals(50.0, (double) arrivalTimes.get(0), precision);
		Assert.assertEquals(116.0, (double) arrivalTimes.get(1), precision);
		Assert.assertEquals(159.0, (double) arrivalTimes.get(2), precision);
		Assert.assertEquals(169.0 + Math.sqrt(125.0), (double) arrivalTimes.get(3), precision);
	}

	@Test
	public void testCacheCost ()
	{
		Assert.assertFalse(vehicle.isCostCalculated());
		vehicle.AssignCustomer(1);
		Assert.assertFalse(vehicle.isCostCalculated());
		vehicle.getCost();
		Assert.assertTrue(vehicle.isCostCalculated());
		vehicle.AssignCustomer(2);
		Assert.assertFalse(vehicle.isCostCalculated());
		vehicle.RemoveCustomer(1);
		Assert.assertFalse(vehicle.isCostCalculated());
		vehicle.getCost();
		Assert.assertTrue(vehicle.isCostCalculated());
		vehicle.ClearListOfCustomersVisiting();
		Assert.assertFalse(vehicle.isCostCalculated());
		vehicle.getCost();
		Assert.assertTrue(vehicle.isCostCalculated());
	}

	@Test
	public void testIsActive ()
	{
		Assert.assertFalse(vehicle.isActive());
		vehicle.AssignCustomer(1);
		Assert.assertTrue(vehicle.isActive());
	}

	@Test
	public void testFillingFunction ()
	{
		vehicle.AssignCustomer(1);
		// alpha = .5
		// Assert.assertEquals(-0.006570644718792868, vehicle.fillingFunction(), precision);
	}

	@Test
	public void testDetourCost ()
	{
		vehicle.AssignCustomer(2);
		Vehicle v = new Vehicle(200, customersFull, new InitialFillingFunction(customersFull, 0.5));
		v.AssignCustomer(2);
		v.AssignCustomer(3);
		Assert.assertEquals(v.getCost() - vehicle.getCost(), vehicle.getDetourCost(3, 2), precision);
	}

	@Test
	public void testDelay ()
	{
		vehicle.AssignCustomer(2);
		Vehicle v = new Vehicle(200, customersFull, new InitialFillingFunction(customersFull, 0.5));
		v.AssignCustomer(2);
		v.AssignCustomer(3);
		Assert.assertEquals(v.checkInTime() - vehicle.checkInTime(), vehicle.getDelay(3, 2), precision);
	}

	@Test
	public void testClone ()
	{
		Vehicle clone = vehicle.clone();
		Assert.assertTrue(clone.equals(vehicle));
		Assert.assertFalse(clone == vehicle);
		Assert.assertEquals(vehicle.fillingFunction(), clone.fillingFunction(), precision);
	}
	
	@Test
	public void testFillingFunctionCanBeCalculatedForNewVehicle ()
	{
		AbstractFillingFunction testFillFunc = new InitialFillingFunction(customersFull, 1.0);
		Vehicle newVehicle = new Vehicle(200, customersFull, testFillFunc, 36);
		newVehicle.fillingFunction();
	}
	
	@Test
	public void testToString ()
	{
		Vehicle newVehicle = new Vehicle(200, customersFull, fillingFunction);
		Assert.assertEquals("{}", newVehicle.toString());
		newVehicle.AssignCustomer(36);
		Assert.assertEquals("{36}", newVehicle.toString());
	}

	// Support methods.
	/**
	 * Adds customer 1, 2, 3, 4 and 5.
	 * 
	 * @return
	 */
	private List<Integer> AddFirstFiveCustomers ()
	{
		List<Integer> listOfCustomers = new ArrayList<Integer>();
		for (int j = 1; j <= 5; j++)
			listOfCustomers.add(j);

		vehicle.AssignCustomer(listOfCustomers);
		return listOfCustomers;
	}
}
