package view;

import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import java.util.Observable;
import java.util.Observer;

import javax.swing.*;

import control.InteractiveController;

import model.*;

public class WorldView implements Observer
{
	private ControlPanel controlPanel;
	private AreaPanel areaPanel;
	private StatusPanel statusPanel;
	private ParameterPanel parameterPanel;

	public WorldView (TabuSearch ts, InteractiveController controller)
	{
		JFrame worldView = new JFrame("VRPTW by Jerôme Baltzersen");
		worldView.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		worldView.setSize(1000, 750);

		JMenuBar menuBar = new JMenuBar();
		worldView.setJMenuBar(menuBar);

		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic(KeyEvent.VK_F);
		menuBar.add(fileMenu);

		JMenuItem load = new JMenuItem("Load data", KeyEvent.VK_L);
		JMenuItem quit = new JMenuItem("Quit", KeyEvent.VK_Q);
		fileMenu.add(load);
		fileMenu.add(quit);
		load.addActionListener(controller);
		quit.addActionListener(controller);
		load.setActionCommand("load");
		quit.setActionCommand("quit");

		controlPanel = new ControlPanel(controller);
		areaPanel = new AreaPanel(ts);
		statusPanel = new StatusPanel(ts);
		parameterPanel = new ParameterPanel();
		controller.setParameterPanel(parameterPanel);

		worldView.add(controlPanel, BorderLayout.NORTH);
		worldView.add(areaPanel, BorderLayout.CENTER);
		worldView.add(statusPanel, BorderLayout.SOUTH);
		worldView.add(parameterPanel, BorderLayout.EAST);

		worldView.setVisible(true);
	}

	public void update (Observable model, Object arg)
	{
		areaPanel.update(model);
		statusPanel.update(model);
	}
}
